import re

from django.conf import settings
from django.core.management.base import BaseCommand
from pathlib import Path


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('-n', '--name', )

    def handle(self, *_, **options):

        project_name = settings.BASE_DIR.parts[-1]
        name = options['name']
        lower_name = re.sub(r'(?<!^)(?=[A-Z])', '_', name).lower()

        # 添加 model 文件
        Path(f"{settings.BASE_DIR}/app/models").mkdir(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/models/__init__.py").touch(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/models/{lower_name}.py").touch(exist_ok=True)
        modelpy = (f"from django.db import models\n"
                   f"from dseagull.models import BaseModel\n\n\n"
                   f"class {name}(BaseModel):\n"
                   f"    pass\n")
        Path(f"{settings.BASE_DIR}/app/models/{lower_name}.py").write_text(modelpy)

        # 添加 serializer 文件
        Path(f"{settings.BASE_DIR}/app/serializers").mkdir(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/serializers/__init__.py").touch(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/serializers/{lower_name}.py").touch(exist_ok=True)
        serializerpy = (f"from rest_framework.serializers import ModelSerializer\n"
                        f"from dseagull.serializers import DateTimeField\n\n"
                        f"from app.models.{lower_name} import {name}\n\n\n"
                        f"class {name}CreateSerializer(ModelSerializer):\n"
                        f"    create_at = DateTimeField(help_text='创建时间', format='%Y-%m-%d %H:%M:%S')  # todo\n\n"
                        f"    class Meta:\n"
                        f"        model = {name}\n"
                        f"        fields = (\n"
                        f"            'id', 'create_at',\n"
                        f"        )\n\n\n"
                        f"class {name}ListSerializer(ModelSerializer):\n"
                        f"    class Meta:\n"
                        f"        model = {name}\n"
                        f"        fields = (\n"
                        f"            'id',\n"
                        f"        )\n")
        Path(f"{settings.BASE_DIR}/app/serializers/{lower_name}.py").write_text(serializerpy)

        # 创建 view 文件
        Path(f"{settings.BASE_DIR}/app/views").mkdir(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/views/__init__.py").touch(exist_ok=True)
        Path(f"{settings.BASE_DIR}/app/views/{lower_name}.py").touch(exist_ok=True)
        viewpy = (f"from django_filters import rest_framework as filters\n"
                  f"from rest_framework.decorators import action\n"
                  f"from rest_framework.versioning import URLPathVersioning\n"
                  f"from rest_framework.viewsets import ModelViewSet\n\n"
                  f"from app.models.{lower_name} import {name}\n"
                  f"from app.serializers.{lower_name} import {name}CreateSerializer\n"
                  f"from app.serializers.{lower_name} import {name}ListSerializer\n"
                  f"from drf.filterset import BaseFilterSet\n\n\n"
                  f"class {name}Filter(BaseFilterSet):\n"
                  f"    create_at = filters.CharFilter(help_text='创建时间', method='filter_datetime', )\n\n"
                  f"    class Meta:\n"
                  f"        model = {name}\n"
                  f"        fields = ('id',)\n\n\n"
                  f"class {name}ViewSet(ModelViewSet):\n"
                  f"    serializer_class = {name}CreateSerializer\n"
                  f"    queryset = {name}.objects.all()\n"
                  f"    filterset_class = {name}Filter\n"
                  f"    versioning_class = URLPathVersioning\n\n"
                  f"    def get_queryset(self):  # todo 请实现数据隔离\n"
                  f"        return super().get_queryset()\n\n"
                  f"    @action(methods=['get'], detail=False, serializer_class={name}ListSerializer, )\n"
                  f"    def paginator_list(self, request, *args, **kwargs):\n"
                  f"        return super().list(request=request, *args, **kwargs)\n")
        Path(f"{settings.BASE_DIR}/app/views/{lower_name}.py").write_text(viewpy)

        ffffff

        # 创建 routers
        # Create a Path object
        routers_file = Path(f"{settings.BASE_DIR}/dashboard/routers.py")
        if not routers_file.exists():
            pass

        # Check if the path exists
        path_exists = routers_file.exists()
        print(f'{path_exists=}')

        xxxxxx

        # routers
        with open(f'{settings.BASE_DIR}/blue/routers.py', 'r+') as f:
            read = f.read()
            f.seek(0)
            f.write(f'from blue.views.{lower_name} import {name}ViewSet\n' + read)

        routerpy = "router.register(f'{version_7}/" + lower_name + f"s', {name}ViewSet, )\n"
        with open(f'{settings.BASE_DIR}/blue/routers.py', 'a+') as f:
            if routerpy not in f.read():
                f.write(routerpy)

        # testcase
        testcasepy = (f"from blue.tests.base import BaseTestCase\n\n\n"
                      f"class {name}TestCase(BaseTestCase):\n"
                      f"    def test_cru(self):\n"
                      f"        r = self.useragency_cdl1.post('/1/v7/{lower_name}s/')\n"
                      f"        self.assertEqual(r.status_code, 201, r.data)\n"
                      f"        r = self.useragency_cdl1.get('/1/v7/{lower_name}s/')\n"
                      f"        self.assertEqual(r.status_code, 200, r.data)\n"
                      f"        self.assertEqual(r.data['paging']['total'], 1, r.data)\n")
        with open(f'{settings.BASE_DIR}/blue/tests/__init__.py', 'a+') as f:
            f.write(f"from blue.tests.{lower_name} import {name}TestCase  # noqa\n")
        with open(f'{settings.BASE_DIR}/blue/tests/{lower_name}.py', 'w') as f:
            f.write(testcasepy)

        print(f'接口文档: http://0.0.0.0:80/lls_docs/{lower_name}s')
